@@ -8,6 +8,7 @@ import ssl
 import sys
 import time
 import zlib
+from enum import Enum
 from pathlib import Path
 from _socket import SHUT_RDWR
 
@@ -21,16 +22,40 @@ from .transformsyslog import (COMPOSE, COMPOSE_BYTES, FACILITY_USER, FORMAT_MY,
 PYPY = hasattr(sys, 'pypy_version_info')
 
 
-ERROR_MSGS = {
-    "no_query": "Error: Not query provided.",
-    "no_auth": "Client dont have key&secret or auth token/jwt",
-    "no_endpoint": "Endpoint 'url' not found"
-}
+class ERROR_MSGS(str, Enum):
+    WRONG_FILE_TYPE = "'%s' is not a valid type to be opened as a file"
+    ADDRESS_TUPLE = "Devo-SenderConfigSSL| address must be a tuple (\"hostname\", int(port))'"
+    WRONG_SSL_CONFIG = "Devo-SenderConfigSSL|Can't create SSL config: %s"
+    CONFIG_FILE_NOT_FOUND = "Error in the configuration, %s is not a file or the path does not exist"
+    CANT_READ_CONFIG_FILE = "Error in the configuration %s can't be read\noriginal error: %s"
+    CONFIG_FILE_PROBLEM = "Error in the configuration, %s problem related to: %s"
+    KEY_NOT_COMPATIBLE_WITH_CERT = "Error in the configuration, the key: %s is not compatible with the cert: %s\noriginal error: %s"
+    CHAIN_NOT_COMPATIBLE_WITH_CERT = "Error in config, the chain: %s is not compatible with the certificate: %s\noriginal error: %s"
+    TIMEOUT_RELATED_TO_AN_INCORRECT_ADDRESS_PORT = "Possible error in config, a timeout could be related to an incorrect address/port: %s\noriginal error: %s"
+    INCORRECT_ADDRESS_PORT = "Error in config, incorrect address/port: %s\noriginal error: %s"
+    CERTIFICATE_IN_ADDRESS_IS_NOT_COMPATIBLE = "Error in config, the certificate in the address: %s is not compatible with: %s"
+    ADDRESS_MUST_BE_A_TUPLE = "Devo-SenderConfigSSL| address must be a tuple '(\"hostname\", int(port))'"
+    CANT_CREATE_TCP_CONFIG = "DevoSenderConfigTCP|Can't create TCP config: %s"
+    PROBLEMS_WITH_SENDER_ARGS = "Problems with args passed to Sender"
+    TCP_CONN_ESTABLISHMENT_SOCKET = "TCP conn establishment socket error: %s"
+    SSL_CONN_ESTABLISHMENT_SOCKET = "SSL conn establishment socket error: %s"
+    PFX_CERTIFICATE_READ_FAILED = "PFX Certificate read failed: %s"
+    SEND_ERROR = "Send error"
+    SOCKET_ERROR = "Socket error: %s"
+    SOCKET_CANT_CONNECT_UNKNOWN_ERROR = "Socket cant connect: unknown error"
+    NO_ADDRESS = "No address"
 
 
 class DevoSenderException(Exception):
     """ Default Devo Sender Exception """
 
+    def __init__(self, message):
+        self.message = message
+        super().__init__(message)
+
+    def __str__(self):
+        return self.message
+
 
 class SenderConfigSSL:
     """
@@ -55,13 +80,12 @@ class SenderConfigSSL:
         Sender
 
     """
+
     def __init__(self, address=None, key=None, cert=None, chain=None,
                  pkcs=None, sec_level=None, check_hostname=True,
                  verify_mode=None, verify_config=False):
         if not isinstance(address, tuple):
-            raise DevoSenderException(
-                "Devo-SenderConfigSSL| address must be a tuple "
-                "'(\"hostname\", int(port))'")
+            raise DevoSenderException(ERROR_MSGS.ADDRESS_TUPLE)
         try:
             self.address = address
             self.key = key
@@ -75,8 +99,7 @@ class SenderConfigSSL:
             self.verify_mode = verify_mode
         except Exception as error:
             raise DevoSenderException(
-                "Devo-SenderConfigSSL|Can't create SSL config: "
-                "%s" % str(error))
+                ERROR_MSGS.WRONG_SSL_CONFIG % str(error)) from error
 
         if self.verify_config:
             self.check_config_files_path()
@@ -94,23 +117,20 @@ class SenderConfigSSL:
         certificates = [self.key, self.chain, self.cert]
         for file in certificates:
             try:
-                if not Path(file).is_file():
+                if not (file.is_file() if isinstance(file, Path) else Path(
+                        file).is_file()):
                     raise DevoSenderException(
-                        "Error in the configuration, "
-                        + file +
-                        " is not a file or the path does not exist")
+                        ERROR_MSGS.CONFIG_FILE_NOT_FOUND % file)
             except IOError as message:
                 if message.errno == errno.EACCES:
                     raise DevoSenderException(
-                        "Error in the configuration "
-                        + file + " can't be read" +
-                        "\noriginal error: " +
-                        str(message))
+                        ERROR_MSGS.CANT_READ_CONFIG_FILE % (
+                            file, str(message))) \
+                        from message
                 else:
                     raise DevoSenderException(
-                        "Error in the configuration, "
-                        + file + " problem related to: " + str(message))
-
+                        ERROR_MSGS.CONFIG_FILE_PROBLEM % (file, str(message))) \
+                        from message
         return True
 
     def check_config_certificate_key(self):
@@ -121,8 +141,8 @@ class SenderConfigSSL:
         :return: Boolean true or raises an exception
         """
 
-        with open(self.cert, "rb") as certificate_file, \
-                open(self.key, "rb") as key_file:
+        with open_file(self.cert, mode="rb") as certificate_file, \
+                open_file(self.key, mode="rb") as key_file:
 
             certificate_raw = certificate_file.read()
             key_raw = key_file.read()
@@ -137,9 +157,9 @@ class SenderConfigSSL:
             context.check_privatekey()
         except SSL.Error as message:
             raise DevoSenderException(
-                "Error in the configuration, the key: " + self.key +
-                " is not compatible with the cert: " + self.cert +
-                "\noriginal error: " + str(message))
+                ERROR_MSGS.KEY_NOT_COMPATIBLE_WITH_CERT % (
+                    self.key, self.cert, str(message)
+                )) from message
         return True
 
     def check_config_certificate_chain(self):
@@ -149,8 +169,8 @@ class SenderConfigSSL:
 
         :return: Boolean true or raises an exception
         """
-        with open(self.cert, "rb") as certificate_file, \
-                open(self.chain, "rb") as chain_file:
+        with open_file(self.cert, mode="rb") as certificate_file, \
+                open_file(self.chain, mode="rb") as chain_file:
 
             certificate_raw = certificate_file.read()
             chain_raw = chain_file.read()
@@ -167,9 +187,9 @@ class SenderConfigSSL:
             store_ctx.verify_certificate()
         except crypto.X509StoreContextError as message:
             raise DevoSenderException(
-                "Error in config, the chain: " + self.chain +
-                " is not compatible with the certificate: " + self.cert +
-                "\noriginal error: " + str(message))
+                ERROR_MSGS.CHAIN_NOT_COMPATIBLE_WITH_CERT % (
+                    self.chain, self.cert, str(message)
+                )) from message
         return True
 
     def check_config_certificate_address(self):
@@ -188,20 +208,20 @@ class SenderConfigSSL:
             connection.connect(self.address)
         except socket.timeout as message:
             raise DevoSenderException(
-                "Possible error in config, a timeout could be related " +
-                "to an incorrect address/port: " + str(self.address) +
-                "\noriginal error: " + str(message))
+                ERROR_MSGS.TIMEOUT_RELATED_TO_AN_INCORRECT_ADDRESS_PORT % (
+                    str(self.address), str(message)
+                )) from message
         except ConnectionRefusedError as message:
             raise DevoSenderException(
-                "Error in config, incorrect address/port: "
-                + str(self.address) +
-                "\noriginal error: " + str(message))
+                ERROR_MSGS.INCORRECT_ADDRESS_PORT % (
+                    str(self.address), str(message)
+                )) from message
         sock.setblocking(True)
         connection.do_handshake()
         server_chain = connection.get_peer_cert_chain()
         connection.close()
 
-        with open(self.chain, "rb") as chain_file:
+        with open_file(self.chain, mode="rb") as chain_file:
             chain = chain_file.read()
             chain_certs = []
             for _ca in pem.parse(chain):
@@ -217,16 +237,15 @@ class SenderConfigSSL:
             return True
 
         raise DevoSenderException(
-            "Error in config, the certificate in the address: "
-            + self.address[0] +
-            " is not compatible with: " +
-            self.chain)
+            ERROR_MSGS.CERTIFICATE_IN_ADDRESS_IS_NOT_COMPATIBLE % (
+                self.address[0], self.chain
+            ))
 
     @staticmethod
     def get_common_names(cert_chain, components_type):
         result = set()
         for temp_cert in cert_chain:
-            for key, value in getattr(temp_cert, components_type)()\
+            for key, value in getattr(temp_cert, components_type)() \
                     .get_components():
                 if key.decode("utf-8") == "CN":
                     result.add(value)
@@ -234,7 +253,7 @@ class SenderConfigSSL:
 
     @staticmethod
     def fake_get_peer_cert_chain(chain):
-        with open(chain, "rb") as chain_file:
+        with open_file(chain, mode="rb") as chain_file:
             chain_certs = []
             for _ca in pem.parse(chain_file.read()):
                 chain_certs.append(
@@ -256,21 +275,19 @@ class SenderConfigTCP:
 
     def __init__(self, address=None):
         if not isinstance(address, tuple):
-            raise DevoSenderException(
-                "Devo-SenderConfigSSL| address must be a tuple "
-                "'(\"hostname\", int(port))'")
+            raise DevoSenderException(ERROR_MSGS.ADDRESS_MUST_BE_A_TUPLE)
         try:
             self.address = address
             self.hostname = socket.gethostname()
             self.sec_level = None
         except Exception as error:
             raise DevoSenderException(
-                "DevoSenderConfigTCP|Can't create TCP config: "
-                "%s" % str(error))
+                ERROR_MSGS.CANT_CREATE_TCP_CONFIG % str(error)) from error
 
 
 class SenderBuffer:
     """Micro class for buffer values"""
+
     def __init__(self):
         self.length = 19500
         self.compression_level = -1
@@ -286,19 +303,23 @@ class Sender(logging.Handler):
     :param con_type: TCP or SSL, default SSL, you can pass it in
     config object too
     :param timeout: timeout for socket
+    :param inactivity_timeout: inactivity timeout for Ingestion balancer, so connection is restarted before reaching
     :param debug: For more info in console/logger output
     :param logger: logger. Default sys.console
     """
-    def __init__(self, config=None, con_type=None,
+
+    def __init__(self, config=None, con_type=None, inactivity_timeout=30,
                  timeout=30, debug=False, logger=None):
         if config is None:
-            raise DevoSenderException("Problems with args passed to Sender")
+            raise DevoSenderException(ERROR_MSGS.PROBLEMS_WITH_SENDER_ARGS)
 
         self.socket = None
         self.reconnection = 0
         self.debug = debug
         self.socket_timeout = timeout
+        self.inactivity_timeout = inactivity_timeout
         self.socket_max_connection = 3600 * 1000
+        self.last_message = int(time.time())
         self.buffer = SenderBuffer()
         self.logging = {}
 
@@ -312,7 +333,7 @@ class Sender(logging.Handler):
         self.logger = logger if logger else \
             get_log(handler=get_stream_handler(
                 msg_format='%(asctime)s|%(levelname)s|Devo-Sender|%(message)s')
-                )
+            )
 
         self._sender_config = config
 
@@ -344,10 +365,11 @@ class Sender(logging.Handler):
         self.socket.settimeout(self.socket_timeout)
         try:
             self.socket.connect(self._sender_config.address)
+            self.last_message = int(time.time())
         except socket.error as error:
             self.close()
             raise DevoSenderException(
-                "TCP conn establishment socket error: %s" % str(error))
+                ERROR_MSGS.TCP_CONN_ESTABLISHMENT_SOCKET % str(error)) from error
 
         self.timestart = int(round(time.time() * 1000))
 
@@ -372,41 +394,38 @@ class Sender(logging.Handler):
         except Exception as error:
             self.close()
             raise DevoSenderException(
-                "PFX Certificate read failed: %s" %
-                str(error))
+                ERROR_MSGS.PFX_CERTIFICATE_READ_FAILED % str(error)) from error
         try:
-            try:
-                if self._sender_config.key is not None \
-                        and self._sender_config.chain is not None \
-                        and self._sender_config.cert is not None:
+            if self._sender_config.key is not None \
+                    and self._sender_config.chain is not None \
+                    and self._sender_config.cert is not None:
 
-                    context = ssl.create_default_context(
-                        cafile=self._sender_config.chain)
+                context = ssl.create_default_context(
+                    cafile=self._sender_config.chain)
 
-                    if self._sender_config.sec_level is not None:
-                        context.set_ciphers(
-                            "DEFAULT@SECLEVEL={!s}"
-                            .format(self._sender_config.sec_level))
+                if self._sender_config.sec_level is not None:
+                    context.set_ciphers(
+                        "DEFAULT@SECLEVEL={!s}"
+                        .format(self._sender_config.sec_level))
 
-                    context.check_hostname = self._sender_config.check_hostname
+                context.check_hostname = self._sender_config.check_hostname
 
-                    if self._sender_config.verify_mode is not None:
-                        context.verify_mode = self._sender_config.verify_mode
+                if self._sender_config.verify_mode is not None:
+                    context.verify_mode = self._sender_config.verify_mode
 
-                    context.load_cert_chain(keyfile=self._sender_config.key,
-                                            certfile=self._sender_config.cert)
-                    self.socket = \
-                        context.wrap_socket(
-                            self.socket,
-                            server_hostname=self._sender_config.address[0]
-                        )
-                else:
-                    self.socket = ssl.wrap_socket(self.socket,
-                                                  cert_reqs=ssl.CERT_NONE)
-            except ssl.SSLError:
-                raise ssl.SSLError
+                context.load_cert_chain(keyfile=self._sender_config.key,
+                                        certfile=self._sender_config.cert)
+                self.socket = \
+                    context.wrap_socket(
+                        self.socket,
+                        server_hostname=self._sender_config.address[0]
+                    )
+            else:
+                self.socket = ssl.wrap_socket(self.socket,
+                                              cert_reqs=ssl.CERT_NONE)
 
             self.socket.connect(self._sender_config.address)
+            self.last_message = int(time.time())
             self.reconnection += 1
             if self.debug:
                 self.logger.debug('Conected to %s|%s'
@@ -417,8 +436,7 @@ class Sender(logging.Handler):
         except socket.error as error:
             self.close()
             raise DevoSenderException(
-                "SSL conn establishment socket error: %s" %
-                str(error))
+                ERROR_MSGS.SSL_CONN_ESTABLISHMENT_SOCKET % str(error)) from error
 
     def info(self, msg):
         """
@@ -511,6 +529,13 @@ class Sender(logging.Handler):
         if self.socket_max_connection < timeit:
             self.close()
             return False
+
+        # If there is no activity (connection or message sent) for an amount of time bigger then the inactivity
+        # timeout, the balancer may have already close the connection. Close it and reconnect.
+        if int(time.time()) - self.last_message > self.inactivity_timeout:
+            self.close()
+            return False
+
         return True
 
     def close(self):
@@ -531,7 +556,7 @@ class Sender(logging.Handler):
             record = Sender.__encode_record(record)
             return b'%d %s' % (len(record), record)
         except Exception as error:
-            raise DevoSenderException(error)
+            raise DevoSenderException(error) from error
 
     @staticmethod
     def __encode_record(record):
@@ -553,11 +578,12 @@ class Sender(logging.Handler):
         for iteration in range(0, total):
             part = record[int(iteration * 4096):
                           int((iteration + 1) * 4096)]
+            self.last_message = int(time.time())
             if self.socket.sendall(part) is not None:
-                raise DevoSenderException("Send error")
+                raise DevoSenderException(ERROR_MSGS.SEND_ERROR)
             sent += len(part)
         if sent == 0:
-            raise DevoSenderException("Send error")
+            raise DevoSenderException(ERROR_MSGS.SEND_ERROR)
         return sent
 
     def send_raw(self, record, multiline=False, zip=False):
@@ -577,8 +603,9 @@ class Sender(logging.Handler):
                     if not multiline and not zip:
                         msg = self.__encode_record(record)
                         sent = len(msg)
+                        self.last_message = int(time.time())
                         if self.socket.sendall(msg) is not None:
-                            raise DevoSenderException("Send error")
+                            raise DevoSenderException(ERROR_MSGS.SEND_ERROR)
                         return 1
                     if multiline:
                         record = self.__encode_multiline(record)
@@ -587,17 +614,17 @@ class Sender(logging.Handler):
                     if sent:
                         return 1
                     return 0
-                except socket.error:
+                except socket.error as error:
                     self.close()
                     raise DevoSenderException(
-                        "Socket error: %s" % str(socket.error))
+                        ERROR_MSGS.SOCKET_ERROR % str(error)) from error
                 finally:
                     if self.debug:
                         self.logger.debug('sent|%d|size|%d|msg|%s' %
                                           (sent, len(record), record))
-            raise Exception("Socket cant connect: unknown error")
+            raise Exception(ERROR_MSGS.SOCKET_CANT_CONNECT_UNKNOWN_ERROR)
         except Exception as error:
-            raise DevoSenderException(error)
+            raise DevoSenderException(error) from error
 
     @staticmethod
     def compose_mem(tag, **kwargs):
@@ -765,7 +792,7 @@ class Sender(logging.Handler):
         address = config.get("address", None)
 
         if not address:
-            raise DevoSenderException("No address")
+            raise DevoSenderException(ERROR_MSGS.NO_ADDRESS)
 
         if not isinstance(address, tuple):
             address = (address, int(config.get("port", 443)))
@@ -810,3 +837,20 @@ class Sender(logging.Handler):
                       severity=severity)
         except Exception:
             self.handleError(record)
+
+
+def open_file(file, mode='r', encoding='utf-8'):
+    """
+    Helper class to open file whenever is provided as `Path` or `str` type
+    :param file File to open
+    :param mode Opening mode
+    :param encoding Encoding of content
+    """
+    if isinstance(file, Path):
+        return file.open(mode=mode,
+                         encoding=encoding if not mode.endswith('b') else None)
+    elif isinstance(file, str):
+        return open(file, mode=mode,
+                    encoding=encoding if not mode.endswith('b') else None)
+    else:
+        raise DevoSenderException(ERROR_MSGS.WRONG_FILE_TYPE % str(type(file)))
